local spawn_charge, spawn_napalm = spawn_charge, spawn_napalm

SOUND_X_CHARGE_BEGIN_IDX = 1
SOUND_X_CHARGE_LOOP_IDX = 2
SOUND_X_CHARGE_MAX_BEGIN_IDX = 3
SOUND_X_CHARGE_MAX_LOOP_IDX = 4
SOUND_X_GIGA_CHARGE_HOLD_IDX = 5
SOUND_X_SHOT1_IDX = 6
SOUND_X_SHOT2_IDX = 7
SOUND_X_SHOT3_IDX = 8
SOUND_X_SHOT_MAX_IDX = 9
SOUND_VILE_SHOT_IDX = 10
SOUND_VILE_CANNON_IDX = 11

local packet_types_funcs = {
    -- Sounds
    ['sound'] = function(m, table)
        local soundtable = {
            [SOUND_X_CHARGE_BEGIN_IDX] = SOUND_X_CHARGE_BEGIN,
            [SOUND_X_CHARGE_LOOP_IDX] = SOUND_X_CHARGE_LOOP,
            [SOUND_X_CHARGE_MAX_BEGIN_IDX] = SOUND_X_CHARGE_MAX_BEGIN,
            [SOUND_X_CHARGE_MAX_LOOP_IDX] = SOUND_X_CHARGE_MAX_LOOP,
            [SOUND_X_GIGA_CHARGE_HOLD_IDX] = SOUND_X_GIGA_CHARGE_HOLD,
            [SOUND_X_SHOT1_IDX] = SOUND_X_SHOT1,
            [SOUND_X_SHOT2_IDX] = SOUND_X_SHOT2,
            [SOUND_X_SHOT3_IDX] = SOUND_X_SHOT3,
            [SOUND_X_SHOT_MAX_IDX] = SOUND_X_SHOT_MAX,
            [SOUND_VILE_SHOT_IDX] = SOUND_VILE_SHOT,
            [SOUND_VILE_CANNON_IDX] = SOUND_VILE_CANNON
        }
        mmx_play_sound(soundtable[table.sound_idx], m.pos, table.intensity)
    end,
    -- Effects
    ['charge'] = function(m, table)
        spawn_charge(m, table.chargeLevel)
    end,
    -- Weapons
    ['saber'] = function(m, table)
        spawn_non_sync_object(id_bhvSaber, E_MODEL_NONE, m.pos.x, m.pos.y, m.pos.z, function(o)
            o.oOwner = gNetworkPlayers[m.playerIndex].globalIndex
            o.parentObj = m.marioObj
            o.oSaberIndex = table.saber_idx
            o.oUnkC0 = table.id
        end)
    end,
    ['raikousen'] = function(m, table)
        spawn_non_sync_object(id_bhvRaikousenLightning, E_MODEL_NONE, m.pos.x, m.pos.y + 80, m.pos.z, function(o)
            o.oOwner = gNetworkPlayers[m.playerIndex].globalIndex
            o.oUnkC0 = table.id
        end)
    end,
    ['napalm'] = function(m, table)
        local grounded = false
        if table.grounded == 1 then
            grounded = true
        end
        spawn_napalm(m, grounded, table.id)
    end,
    ['shot'] = function(m, table)
        local shotmodeltable = {
            [1] = E_model_shot1,
            [2] = E_model_vulcan_shot,
            [3] = E_model_shot2,
            [4] = E_model_shot3,
            [5] = E_model_cannon_shot,
            [6] = E_model_shot_max,
            [7] = E_model_giga_pink_shot,
            [8] = E_model_shot3_particle,
        }
        spawn_non_sync_object(id_bhvShot, shotmodeltable[table.shotModel], table.posX, table.posY, table.posZ, function(o)
            o.oOwner = gNetworkPlayers[m.playerIndex].globalIndex
            o.oMoveAngleYaw = table.oMoveAngleYaw
            o.oOwnerChar = table.oOwnerChar
            o.oShotType = table.oShotType
            o.oMmxAtkDmg = table.oMmxAtkDmg
            o.oShotOffset = table.oShotOffset
            o.oChargeRot = table.oChargeRot
            o.oShotDestroyNextFrame = 0
            o.oUnkC0 = table.id
        end)
    end,
    ['shot_id_sync'] = function(m, table)
        local mmx = gMmxStates[m.playerIndex]
        if table.remove == 1 then
            mmx.active_shot_hitboxes[table.id] = nil
        else
            mmx.active_shot_hitboxes[table.id] = {}
        end
    end,
    ['shot_saber_sync'] = function(m, table)
        local mmx = gMmxStates[m.playerIndex]
        if table.remove == 1 then
            mmx.zero_saber_active_hitboxes[table.id] = nil
        else
            mmx.zero_saber_active_hitboxes[table.id] = {}
        end
    end,
    ['saber_id_sync'] = function(m, table)
        local mmx = gMmxStates[m.playerIndex]
        if table.remove == 1 then
            mmx.zero_saber_active_hitboxes[table.id] = nil
        else
            mmx.zero_saber_active_hitboxes[table.id] = {}
        end
    end,
    ['explosion_id_sync'] = function(m, table)
        local mmx = gMmxStates[m.playerIndex]
        if table.remove == 1 then
            mmx.active_explosion_hitboxes[table.id] = nil
        else
            mmx.active_explosion_hitboxes[table.id] = {}
        end
    end
}

local on_receive_spawn_data = function(table)
    local np = network_player_from_global_index(table.m)
    local m = gMarioStates[np.localIndex]
    if (active_player(m)) then
        packet_types_funcs[table.type](m, table)
    end
end

hook_event(HOOK_ON_PACKET_RECEIVE, on_receive_spawn_data)